#(c)www.stani.be

import blenpy

INFO=blenpy.INFO.copy()

INFO['description']=\
"""This module (gui) provides a wrapper class 'gui' to automate the building of
user interface in Blender."""

__doc__=INFO['doc']%INFO

#---Import modules--------------------------------------------------------------
try:
    from Blender import BGL,Draw
except ImportError:
    print "This module must be run within Blender"

#---Constants-------------------------------------------------------------------
EXECUTE="#@"

#---gui-------------------------------------------------------------------------
class Gui:
    """Class for building Blender gui's.

    Relevant attributes:
        mouseX,mouseY:      current mouse position.

    See text method for how to apply colored texts or execute Python code with %s.
    """%EXECUTE
#---Private---------------------------------------------------------------------
    def __init__(self,draw=0,bgColor=(0.5,0.5,0.5,0.0),escape=1,mouse=1,start=0,
                 control_width=150,control_height=18,textColor=(0,0,0),
                 tooltip='www.stani.be',originX=5,originY=1,safeId=10,
                 textShift=5):
        """Use this class as a subclass and define your own 'draw' method.

        Parameters:
        draw:           drawing function of the gui
        bgColor:        r,g,b,a; if 0 transparent
        escape:         creates automatically an escape function
        mouse:          automatic mouse tracking to self.mouseX & self.mouseY
        start:          start immediately
        originX,originY:where the gui starts to be drawn
        tooltip:        default tooltip when there is nothing specified.
        """
        if draw:self.draw=draw
        self.bgColor=bgColor
        self.escape=escape
        self.mouse=mouse
        self.control_width=control_width
        self.control_height=control_height
        self.textColor=textColor
        self.tooltip=tooltip
        self.originX=originX
        self.originY=originY
        self.safeId=safeId#minimum id value,because a menu always calls id 4
        self.textShift=textShift

        self._controlId=-1#value of id of last called control,not used
        self.mouseX=self.mouseY=0

        if start: self.start()

    def _color(self,red=0,green=0,blue=0):
        """Sets the color, but doesn't save as text color."""
        BGL.glColor3f(red,green,blue)

    def _control(self,id):
        """Retrieve the actions of the control events (buttons,menus).
        Private function used by register.
        """
        self._controlId=id #maybe useful for tracking
        if 0<=id-self.safeId<=len(self._controls):
            self._controls[id-self.safeId]()

    def _key(self,key,val):
        """Retrieve the actions of the key events.
        Private function used by register.
        """
        if key in self._keys.keys():self._keys[key](val)

    def _mouseX(self,val):
        """pdate mouse x coordinate."""
        self.mouseX=val

    def _mouseY(self,val):
        """pdate mouse y coordinate."""
        self.mouseY=val

    def _prepare(self,typ,id,action,initial,x,y,dx,dy,width,height,tooltip):
        """Private helper function for controls as strings, menus,..."""
        if not id:id=typ+'_'+str(self.count[typ])
        if not action:action=self.nothing
        if self._shown:initial=self.value[id].val
        elif initial==-1:
            if not id:
                if typ=='string':initial=''
                elif typ=='menu':initial=1
            else:initial=self.value[id].val
        initial=Draw.Create(initial)
        tooltip=self.cursor(x,y,dx,dy,width,height,tooltip)
        self.count[typ]+=1
        return (id,action,initial,tooltip)
#---General---------------------------------------------------------------------
    def control(self,action):
        """Helper function for buttons, menus and ..."""
        self._controls.append(action)
        return len(self._controls)+self.safeId-1

    def cursor(self,x=-1,y=-1,dx=-1,dy=-1,width=0,height=0,tooltip=0):
        """Moves the cursor absolutely(x,y) or relatively (dx,dy)."""
        if x>-1:self._x=self.originX+x
        if y>-1:self._y=self.originY+y
        if dx>-1:self._x+=dx+self.control_width
        if dy>-1:self._y+=dy+self.control_height
        if width:self.control_width=width
        if height:selfbutton_height=height
        if type(tooltip).__name__=='str':return tooltip
        else:return self.tooltip

    def exit(self):
        """Exists gui."""
        Draw.Exit()

    def key(self,key=Draw.ESCKEY,action=Draw.Exit):
        """"Register a key with an action"""
        self._keys[key]=action

    def display(self):
        """Resets all values and draws gui."""
        self.count={'button':0,'menu':0,'string':0,'text':0}
        self._controls=[]
        self._keys={}
        self._x=self.originX
        self._y=self.originY
        if self.bgColor:self.clear(*self.bgColor)
        if self.escape:self.key()
        if self.mouse:
            self.key(Draw.MOUSEX,self._mouseX)
            self.key(Draw.MOUSEY,self._mouseY)
        self.draw()
        self._shown=1
    def nothing(self):
        """Do nothing(pass),usefull for unimplemented buttons."""
        pass
    def redraw(self):
        """Redraw gui."""
        Draw.Redraw()
    def start(self):
        """Starts the gui."""
        self._shown=0
        self.value={}
        for key in self.value.keys():self.value[key]=Draw.Create(self.value[key])
        Draw.Register(self.display,self._key,self._control)
#---Drawing Methods------------------------------------------------------------
    def clear(self,r=0.5,g=0.5,b=0.5,a=0.0):
        """Clears the screen with the rgba color"""
        BGL.glClearColor(r,g,b,a)
        BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)

    def button(self,label='',action=0,x=-1,y=-1,dx=-1,dy=-1,width=0,height=0,tooltip=0):
        """Draws a button control, exit is the default action."""
        if action==0:action=self.nothing
        tooltip=self.cursor(x,y,dx,dy,width,height,tooltip)
        self.count['button']+=1
        return Draw.Button(label,self.control(action),self._x,self._y,
                           self.control_width,self.control_height,tooltip)

    def color(self,red=0,green=0,blue=0):
        """Sets the text color."""
        self.textColor=(red,green,blue)
        self._color(red,green,blue)

    def menu(self,label='',id='',action=0,x=-1,y=-1,dx=-1,dy=-1,
             width=0,height=0,initial=-1,tooltip=0):
        """Draw a menu control."""
        if type(label).__name__=='list':label='|'.join(label)
        id,action,initial,tooltip=self._prepare('menu',id,action,initial,x,y,dx,dy,width,height,tooltip)
        self.value[id]=Draw.Menu(label,self.control(action),self._x,self._y,
                                 self.control_width,self.control_height,initial.val,
                                 tooltip)
        return self.value[id]

    def rectangle(self,x0,y0,x1,y1,color=(0.5,0.5,0.5),border=0,borderColor=(0,0,0)):
        """Fills a rectangle with a certain color with an optional border.

        Parameters:
            x0,y0,x1,y1:    coordinates of the lower left and upper right corner
            color:          color with which the rectangle is filled
            border:         border width in pixels
                            0 for no border
                            positive values for expanding border
                            negative values for contracting border
        """
        if border:
            self._color(*borderColor)
            if border>0:
                BGL.glRectf(x0-border,y0-border,x1+border,y1+border)
                border=0
            else:
                BGL.glRectf(x0,y0,x1,y1)
        self._color(*color)
        BGL.glRectf(x0-border,y0-border,x1+border,y1+border)
        self._color(*self.textColor)

    def string(self,label='',id='',action=0,x=-1,y=-1,dx=-1,dy=-1,
             width=0,height=0,initial=-1,length=255,tooltip=0):
        """Draw a string control."""
        id,action,initial,tooltip=self._prepare('string',id,action,initial,x,y,dx,dy,width,height,tooltip)
        self.value[id]=Draw.String(label,self.control(action),self._x,self._y,
                                   self.control_width,self.control_height,initial.val,
                                   length,tooltip)
        return self.value[id]

    def text(self,label='',x=-1,y=-1,dx=-1,dy=-1,width=0,height=0,color=0,bgOffset=(0,0,0,0),bgColor=0):
        """Draws a text on the gui.

        Special attribute: bgOffset
        This specifies in relative units the rectangle for the background
        rectangle.

        Important: Everything after '%s' will be executed as Python code.
        This makes it possible to apply advanced layout techniques.

        For example:

            'blabla#@color=(1,0,0)#@bgColor=self.bgColor'
            This will print the text 'blabla' in a red color on a background
            color, defined by self.bgColor.
        """
        split=label.split(EXECUTE)
        if len(split)>1:
            label=split[0]
            for instr in split[1:]:exec(instr)
        self.cursor(x,y,dx,dy,width,height)
        if bgColor:
            self.rectangle(self._x+bgOffset[0],self._y+bgOffset[1],
                           self._x+self.control_width+bgOffset[2],
                           self._y+self.control_height+bgOffset[3],
                           bgColor)
        if color:self.color(*color)
        BGL.glRasterPos2d(self._x, self._y+self.textShift)
        self.count['text']+=1
        return Draw.Text(label)